@@ -3,7 +3,7 @@ package MVC::Neaf::Request;
 use strict;
 use warnings;
 
-our $VERSION = 0.24;
+our $VERSION = 0.25;
 
 =head1 NAME
 
@@ -49,11 +49,14 @@ use Encode;
 use HTTP::Headers::Fast;
 use Time::HiRes ();
 use Sys::Hostname ();
-use Digest::MD5 qw(md5_base64);
+use Digest::MD5 qw(md5);
 
-use MVC::Neaf::Util qw(http_date run_all_nodie canonize_path);
+use MVC::Neaf::Util qw( http_date run_all_nodie canonize_path encode_b64 );
 use MVC::Neaf::Upload;
 use MVC::Neaf::Exception;
+use MVC::Neaf::Route::Null;
+
+our %allow_helper;
 
 =head2 new()
 
@@ -70,7 +73,7 @@ Restrictions MAY BE added in the future though.
 sub new {
     my ($class, %args) = @_;
 
-    # TODO 0.20 restrict params
+    # TODO 0.30 restrict params
     return bless \%args, $class;
 };
 
@@ -207,14 +210,16 @@ sub path {
 
 =head2 route()
 
-A L<MVC::Neaf::Route> object that this request is being dispatched to,
-or the root of the routing tree if none yet.
+A L<MVC::Neaf::Route> object that this request is being dispatched to.
+
+If request is not inside an application, returns a L<MVC::Neaf::Route::Null>
+instead.
 
 =cut
 
 sub route {
     my $self = shift;
-    return $self->{route};
+    return $self->{route} || MVC::Neaf::Route::Null->new;
 };
 
 =head2 set_path()
@@ -433,6 +438,8 @@ The regular expression is applied to the WHOLE string,
 from beginning to end, not just the middle.
 Use '.*' if you really trust the data.
 
+Dies if regular expression didn't match and the route has the C<strict> flag.
+
 B<[EXPERIMENTAL]> If C<param_regex> hash was given during route definition,
 C<$regex> MAY be omitted for params that were listed there.
 This feature is not stable yet, though. Use with care.
@@ -473,7 +480,9 @@ sub param {
     return $default if !defined $value;
     return $value   if  $value =~ /^(?:$regex)$/s;
 
-    # TODO 0.30 die 422 if strict mode on
+    if ($self->route->strict) {
+        die 422; # TODO 0.30 configurable die message
+    };
     return $default;
 };
 
@@ -490,6 +499,8 @@ If method is GET or HEAD, identical to C<param>.
 Otherwise would return the parameter from query string,
 AS IF it was a GET request.
 
+Dies if regular expression didn't match and the route has the C<strict> flag.
+
 Multiple values are deliberately ignored.
 
 See L<CGI>.
@@ -520,9 +531,13 @@ sub url_param {
     my $value = $self->{url_param_hash}{$name};
 
     # this is copypaste from param(), do something (or don't)
-    return (defined $value and $value =~ /^(?:$regex)$/s)
-        ? $value
-        : $default;
+    return $default if !defined $value;
+    return $value   if  $value =~ /^(?:$regex)$/s;
+
+    if ($self->route->strict) {
+        die 422; # TODO 0.30 configurable die message
+    };
+    return $default;
 };
 
 =head2 multi_param( ... )
@@ -537,6 +552,7 @@ Get a multiple value GET/POST parameter as a C<@list>.
 The name generally follows that of newer L<CGI> (4.08+).
 
 ALL values must match the regex, or an empty list is returned.
+Dies if strict mode if enabled for route and there is a mismatch.
 
 B<[EXPERIMENTAL]> If C<param_regex> hash was given during route definition,
 C<$regex> MAY be omitted for params that were listed there.
@@ -562,7 +578,12 @@ sub multi_param {
     ];
 
     # ANY mismatch = no go. Replace with simple grep if want filter EVER.
-    return (grep { !/^(?:$regex)$/s } @$ret) ? () : @$ret;
+    if (grep { !/^(?:$regex)$/s } @$ret) {
+        die 422 if $self->route->strict;
+        return ();
+    } else {
+        return @$ret;
+    };
 };
 
 =head2 set_param( ... )
@@ -628,11 +649,11 @@ sub form {
             || $self->_croak("Unknown form name $validator");
     };
 
-    if (ref $validator eq 'CODE') {
-        return $validator->( $self->_all_params );
-    } else {
-        return $validator->validate( $self->_all_params );
-    };
+    my $result = (ref $validator eq 'CODE')
+        ? $validator->( $self->_all_params )
+        : $validator->validate( $self->_all_params );
+
+    return $result;
 };
 
 =head2 get_form_as_list( ... )
@@ -786,6 +807,8 @@ The regular expression is applied to the WHOLE string,
 from beginning to end, not just the middle.
 Use '.*' if you really need none.
 
+Dies if regular expression didn't match and the route has the C<strict> flag.
+
 =cut
 
 sub get_cookie {
@@ -807,7 +830,12 @@ sub get_cookie {
     my $value = $self->{neaf_cookie_in}{ $name };
     return $default unless defined $value;
 
-    return $value =~ /^$regex$/ ? $value : $default;
+    return $value if $value =~ /^$regex$/;
+
+    if ($self->route->strict) {
+        die 422; # TODO 0.30 configurable die message
+    };
+    return $default;
 };
 
 =head2 set_cookie( ... )
@@ -1486,32 +1514,23 @@ sub clear {
 
 =head2 id()
 
-Lazily fetch unique request id. These are guaranteed to be unique
-on a given machine within a reasonable timeframe.
+Fetch unique request id. If none has been set yet (see L</set_id>),
+use L</make_id> method to create one.
 
-Current id-generation mechanism involves URI-safe md5_base64 C<[-_]>,
-but this MAY change in the future.
+The request id is present in both log messages from Neaf
+and default error pages, making it easier to establish link
+between the two.
 
-B<[CAUTION]> Don't use this id for anything secure.
-Use L<MVC::Neaf::X::Session>'s ids instead.
-This one is just for information.
+Custom ids can be provided, see L</make_id> below.
 
 =cut
 
-# TODO 0.30 provide a configurable mechanism to generate ids
-my $host = Sys::Hostname::hostname();
-my $lastid = 0;
 sub id {
     my $self = shift;
 
     # We don't really need to protect anything here
     # Just avoid accidental matches for which md5 seems good enough
-    return $self->{id} ||= do {
-        my $str = md5_base64( join ".", $host, $$, Time::HiRes::time, ++$lastid );
-        $lastid = 0 unless $lastid < 4_000_000_000;
-        $str =~ tr/+\//-_/;
-        $str;
-    };
+    return $self->{id} ||= $self->make_id;
 };
 
 =head2 set_id( ... )
@@ -1629,6 +1648,79 @@ sub DESTROY {
         if (exists $self->{response}{postponed});
 };
 
+=head1 METHODS THAT CAN BE OVERRIDDEN
+
+Generally Neaf allows to define custom Request methods restricted to certain
+path & method combinations.
+
+The following methods are available by default, but can be overridden
+via the helper mechanism.
+
+For instance,
+
+    use MVC::Neaf;
+
+    my $id;
+    neaf helper => make_id => sub { $$ . "_" . ++$id };
+    neaf helper => make_id => \&unique_secure_base64, path => '/admin';
+
+=cut
+
+sub _helper_fallback {
+    my ($name, $impl) = @_;
+
+    my $code = sub {
+        my $self = shift;
+        my $todo = $self->route && $self->route->helpers->{$name} || $impl;
+        $todo->( $self, @_ );
+    };
+
+    $allow_helper{$name}++;
+
+    no strict 'refs'; ## no critic
+    use warnings FATAL => qw(all);
+    *$name = $code;
+};
+
+=head2 make_id
+
+Create unique request id, e.g. for logging context.
+This is called by L</id> if the id has not yet been set.
+
+By default, generates MD5 checksum based on time, hostname, process id, and a
+sequential number and encodes as URL-friendly base64.
+
+B<[CAUTION]> This should be unique, but may not be secure.
+Use L<MVC::Neaf::X::Session> if you need something to rely upon.
+
+=cut
+
+my $host = Sys::Hostname::hostname();
+my $lastid = 0;
+_helper_fallback( make_id => sub {
+    $lastid = 0 if $lastid > 4_000_000_000;
+    encode_b64( md5( join "#", $host, $$, Time::HiRes::time, ++$lastid ) );
+} );
+
+=head2 log_message
+
+    $req->log_message( $level => $message );
+
+By default would print uppercased level, $request->id, route,
+and the message itself.
+
+Levels are not restricted whatsoever.
+Suggested values are "fatal", "error", "warn", "debug".
+
+=cut
+
+_helper_fallback( log_message => sub {
+    my ($req, $level, @mess) = @_;
+
+    warn sprintf "%s req_id=%s [%s]: %s"
+        , uc $level, $req->id, $req->_where, join " ", @mess;
+});
+
 =head1 DRIVER METHODS
 
 The following methods MUST be implemented in every Request subclass
@@ -1763,21 +1855,6 @@ sub header_in_keys {
     $head->header_field_names;
 };
 
-=head2 upload( ... )
-
-B<[DEPRECATED]> Same as upload_raw, but issues a warning.
-Use upload_utf8() for text files, or upload_raw() for binary ones.
-
-=cut
-
-# TODO 0.25 kill
-sub upload {
-    my ($self, $name) = @_;
-
-    carp "NEAF: req->upload() is DEPRECATED, use upload_utf8 or upload_raw instead";
-    return $self->_upload( id => $name, utf8 => 0 );
-};
-
 =head2 endpoint_origin
 
 Returns file:line where controller was defined.
@@ -1793,60 +1870,6 @@ sub endpoint_origin {
     return join " line ", @{ $self->{route}{caller} }[1,2];
 };
 
-=head2 set_full_path( ... )
-
-=over
-
-=item * C<$req-E<gt>set_full_path( $path )>
-
-=item * C<$req-E<gt>set_full_path( $script_name, $path_info )>
-
-=back
-
-Set new path elements which will be returned from this point onward.
-
-Also updates path() value so that path = script_name + path_info
-still holds.
-
-C<set_full_path(undef)> resets C<script_name> to whatever returned
-by the underlying driver.
-
-Returns self.
-
-B<[DEPRECATED]> Use C<set_path()> and C<set_path_info()> instead.
-
-=cut
-
-# TODO 0.25 kill it
-sub set_full_path {
-    my ($self, $script_name, $path_info) = @_;
-
-    carp "NEAF: set_full_path() is DEPRECATED and will be removed in 0.25";
-
-    if (!defined $script_name) {
-        $script_name = $self->do_get_path;
-    };
-
-    # CANONIZE
-    $script_name =~ s#^/*#/#;
-    $self->{prefix} = $script_name;
-
-    if (defined $path_info) {
-        # Make sure path_info always has a slash if nonempty
-        $path_info =~ s#^/+##;
-        $self->{postfix} = Encode::is_utf8($path_info)
-                ? $path_info
-                : decode_utf8(uri_unescape($path_info));
-    } elsif (!defined $self->{postfix}) {
-        $self->{postfix} = '';
-    };
-    # assert $self->{postfix} is defined by now
-
-    $self->{path} = "$self->{prefix}"
-        .(length $self->{postfix} ? "/$self->{postfix}" : '');
-    return $self;
-};
-
 =head2 get_form_as_hash( ... )
 
 =over
