@@ -3,21 +3,22 @@ package MVC::Neaf::Request::Apache2;
 use strict;
 use warnings;
 
-our $VERSION = 0.24;
+our $VERSION = 0.25;
 
 =head1 NAME
 
 MVC::Neaf::Request::Apache2 - Apache2 (mod_perl) driver for Not Even A Framework.
 
-=head1 DESCRIPTION
+=head1 WARNING
 
-Apache2 request that will invoke MVC::Neaf core functions from under mod_perl.
+B<This module is DEPRECATED and will be removed in Neaf 0.30.>
 
-Much to the author's disgrace, this module currently uses
-BOTH Apache2::RequestRec and Apache2::Request from libapreq.
+Use L<Plack::Handler::Apache2> instead.
 
 =head1 SYNOPSIS
 
+Apache2 request that will invoke MVC::Neaf core functions from under mod_perl.
+
 The following apache configuration should work with this module:
 
     LoadModule perl_module        modules/mod_perl.so
@@ -68,6 +69,10 @@ BEGIN {
     };
 };
 
+# TODO 0.30 remove the whole module
+carp __PACKAGE__." is DEPRECATED and will be REMOVED in 0.30. Use Plack::Handler::Apache2 instead."
+    if $ENV{MOD_PERL};
+
 use MVC::Neaf;
 use parent qw(MVC::Neaf::Request);
 
@@ -270,8 +275,6 @@ sub do_write {
     return $self->{driver_raw}->print( $data );
 };
 
-# TODO implement do_close, too!
-
 =head2 handler( $apache_request )
 
 A valid Apache2/mod_perl handler.
