@@ -2,7 +2,7 @@ package MVC::Neaf::Route::Null;
 
 use strict;
 use warnings;
-our $VERSION = '0.2603_03';
+our $VERSION = 0.24;
 
 =head1 NAME
 
@@ -39,14 +39,7 @@ sub new {
     my $class = shift;
 
     return $class->SUPER::new(
-        method  => 'GET',
-        path    => '/',
-        code    => $nobody_home,
-        default => {},
-        hooks   => {},
-        helpers => {},
-        @_
-    );
+        method => 'GET', path => '/', code => $nobody_home, @_ );
 };
 
 =head2 post_setup
@@ -62,7 +55,7 @@ sub post_setup {
 
 Returns C<path> argument given to C<new()>, defaults to C<'/'>.
 
-=head2 method
+=head2 path
 
 Returns C<method> argument given to C<new()>, defaults to C<'GET'>.
 
@@ -78,14 +71,6 @@ returns "strict" parameter.
 
 Returns a "die 404" function by default.
 
-=head2 hooks
-
-Returns an empty hash by default.
-
-=head2 default
-
-Returns anempty hash by default.
-
 =cut
 
 make_getters (
@@ -93,9 +78,6 @@ make_getters (
     method   => 1,
     code     => 1,
     strict   => 1,
-    default  => 1,
-    hooks    => 1,
-    helpers  => 1,
 );
 
 =head2 get_form
@@ -117,6 +99,16 @@ sub get_view {
     croak "No view '$name' in null route";
 };
 
+=head2 helpers
+
+Returns empty hash - no helpers by default.
+
+=cut
+
+sub helpers {
+    +{};
+};
+
 =head2 parent
 
 Returns nothing, as this route isn't attached to any application.
