@@ -2,7 +2,7 @@ package MVC::Neaf::Util;
 
 use strict;
 use warnings;
-our $VERSION = 0.24;
+our $VERSION = 0.25;
 
 =head1 NAME
 
@@ -20,11 +20,14 @@ This module optionally exports anything it has.
 =cut
 
 use Carp;
+use MIME::Base64 3.11;
+
 use parent qw(Exporter);
 our @EXPORT_OK = qw(
     canonize_path http_date path_prefixes rex run_all run_all_nodie
-    JSON encode_json decode_json
-    maybe_list supported_methods extra_missing
+    JSON encode_json decode_json encode_b64 decode_b64
+    extra_missing make_getters maybe_list
+    supported_methods
 );
 our @CARP_NOT;
 
@@ -42,7 +45,7 @@ Convert '////fooo//bar/' to '/foo/bar' and '//////' to either '' or '/'.
 sub canonize_path {
     my ($path, $want_slash) = @_;
 
-    $path =~ s#/+#/#g;
+    $path =~ s#//+#/#g;
     if ($want_slash) {
         $path =~ s#/$##;
         $path =~ s#^/*#/#;
@@ -54,6 +57,34 @@ sub canonize_path {
     return $path;
 };
 
+=head2 decode_b64
+
+Decode unpadded URL-friendly base64.
+Also works on normal one.
+
+See L<MIME::Base64/decode_base64url>.
+
+=cut
+
+sub decode_b64 {
+    my $str = shift;
+
+    $str =~ tr#-_#+/#;
+    return MIME::Base64::decode_base64($str);
+};
+
+=head2 encode_b64
+
+Encode data as unpadded URL-friendly base64 - with C<-> for 62 and C<_> for 63.
+C<=> signs are removed.
+
+See L<MIME::Base64/encode_base64url>.
+
+=cut
+
+sub encode_b64;
+*encode_b64 = \&MIME::Base64::encode_base64url;
+
 =head2 extra_missing
 
     extra_missing( \%input, \%allowed, \@required )
@@ -107,6 +138,55 @@ sub http_date {
         , $week[$date[6]], $date[3], $month[$date[4]], 1900+$date[5], @date[2,1,0]);
 };
 
+=head2 make_getters
+
+Create dumb accessors in the calling class from hash.
+Keys are method names.
+
+Key in the object is hash value if it's an identifier,
+or just method name otherwise:
+
+    package My::Class;
+
+    # (declare constructor somehow)
+    make_getters (
+        foo => bar,
+        baz => 1,
+        quux => '',
+    );
+
+    # ...
+
+    my $obj = My::Class->new;
+
+    $obj->foo;  # {bar}
+    $obj->baz;  # {baz}
+    $obj->quux; # {quux}
+
+=cut
+
+# TODO 0.30 use Class::XSAccessor or smth
+sub make_getters {
+    my %which = @_;
+
+    my $pkg = caller;
+
+    foreach (keys %which) {
+        my $method = $_;
+        my $key = $which{$method};
+        $key = $method unless defined $key and $key =~ /^[a-z_][a-z_0-9]*$/i;
+
+        my $sub = sub {
+            $_[0]->{$key};
+        };
+
+        use warnings FATAL => 'all';
+        no strict 'refs'; ## no critic
+
+        *{ $pkg."::".$method } = $sub;
+    };
+};
+
 =head2 maybe_list
 
     maybe_list( $value, @defaults )
