@@ -2,7 +2,7 @@ package MVC::Neaf::X::Form::Data;
 
 use strict;
 use warnings;
-our $VERSION = 0.24;
+our $VERSION = 0.25;
 
 =head1 NAME
 
@@ -25,7 +25,8 @@ This class is not expected to be created and used directly.
 
 =cut
 
-use Digest::SHA qw(sha1_base64);
+use Digest::SHA qw(sha1);
+use MVC::Neaf::Util qw( encode_b64 );
 use URI::Escape;
 
 use parent qw(MVC::Neaf::X);
@@ -198,7 +199,7 @@ sub sign {
 # A weak ad-hoc HMAC. Use a better one...
 sub _default_sign {
     my ($data, $key) = @_;
-    return sha1_base64( join "?", $key, $data, $key );
+    return encode_b64( sha1( join "?", $key, $data, $key ) );
 };
 
 =head1 LICENSE AND COPYRIGHT
